`include "DecodeMacros.v"

module Control(
    Instruction,

    COP0Read,
    COP0Write,
    Operand0Select,
    Operand1Select,
    ALUOpcode,
    PCSelect,
    MemoryRead,
    MemoryWrite,
    WriteBackSelect
);

`include "ControlConstants.v"
`include "ISAConstants.v"

input       [31:0]                  Instruction;

output                              COP0Read;
output                              COP0Write;
output                              Operand0Select;
output      [1:0]                   Operand1Select;
output      [4:0]                   ALUOpcode;
output      [1:0]                   PCSelect;
output                              MemoryRead;
output                              MemoryWrite;
output      [1:0]                   WriteBackSelect;

reg         [15:0]                  ctrl;

assign COP0Read         = ctrl[15:15];
assign COP0Write        = ctrl[14:14];
assign Operand0Select   = ctrl[13:13];
assign Operand1Select   = ctrl[12:11];
assign ALUOpcode        = ctrl[10:6];
assign PCSelect         = ctrl[5:4];
assign MemoryRead       = ctrl[3:3];
assign MemoryWrite      = ctrl[2:2];
assign WriteBackSelect  = ctrl[1:0];

always@(*) begin
    case(`OPCODE(Instruction))
        SPECIAL: begin
            case(`FUNCT(Instruction))
                SLL:        ctrl = {1'b0, 1'b0, OP0SELSHAMT, OP1SELRT, ALUSLL,   PCSELPLUS4,  1'b0, 1'b0, WBSELRD};
                SRL:        ctrl = {1'b0, 1'b0, OP0SELSHAMT, OP1SELRT, ALUSRL,   PCSELPLUS4,  1'b0, 1'b0, WBSELRD};
                SRA:        ctrl = {1'b0, 1'b0, OP0SELSHAMT, OP1SELRT, ALUSRA,   PCSELPLUS4,  1'b0, 1'b0, WBSELRD};
                SLLV:       ctrl = {1'b0, 1'b0, OP0SELRS,    OP1SELRT, ALUSLL,   PCSELPLUS4,  1'b0, 1'b0, WBSELRD};
                SRLV:       ctrl = {1'b0, 1'b0, OP0SELRS,    OP1SELRT, ALUSRL,   PCSELPLUS4,  1'b0, 1'b0, WBSELRD};
                SRAV:       ctrl = {1'b0, 1'b0, OP0SELRS,    OP1SELRT, ALUSRL,   PCSELPLUS4,  1'b0, 1'b0, WBSELRD};
                JR:         ctrl = {1'b0, 1'b0, 1'bx,        2'bx,     5'bx,     PCSELJMPREG, 1'b0, 1'b0, WBSELNONE};
                JALR:       ctrl = {1'b0, 1'b0, 1'bx,        OP1SELPC, ALUPASS1, PCSELJMPREG, 1'b0, 1'b0, WBSELRD};
                ADD:        ctrl = {1'b0, 1'b0, OP0SELRS,    OP1SELRT, ALUADD,   PCSELPLUS4,  1'b0, 1'b0, WBSELRD};
                ADDU:       ctrl = {1'b0, 1'b0, OP0SELRS,    OP1SELRT, ALUADDU,  PCSELPLUS4,  1'b0, 1'b0, WBSELRD};
                SUB:        ctrl = {1'b0, 1'b0, OP0SELRS,    OP1SELRT, ALUSUB,   PCSELPLUS4,  1'b0, 1'b0, WBSELRD};
                SUBU:       ctrl = {1'b0, 1'b0, OP0SELRS,    OP1SELRT, ALUSUBU,  PCSELPLUS4,  1'b0, 1'b0, WBSELRD};
                AND:        ctrl = {1'b0, 1'b0, OP0SELRS,    OP1SELRT, ALUAND,   PCSELPLUS4,  1'b0, 1'b0, WBSELRD};
                OR:         ctrl = {1'b0, 1'b0, OP0SELRS,    OP1SELRT, ALUOR,    PCSELPLUS4,  1'b0, 1'b0, WBSELRD};
                XOR:        ctrl = {1'b0, 1'b0, OP0SELRS,    OP1SELRT, ALUXOR,   PCSELPLUS4,  1'b0, 1'b0, WBSELRD};
                NOR:        ctrl = {1'b0, 1'b0, OP0SELRS,    OP1SELRT, ALUNOR,   PCSELPLUS4,  1'b0, 1'b0, WBSELRD};
                SLT:        ctrl = {1'b0, 1'b0, OP0SELRS,    OP1SELRT, ALUSLT,   PCSELPLUS4,  1'b0, 1'b0, WBSELRD};
                SLTU:       ctrl = {1'b0, 1'b0, OP0SELRS,    OP1SELRT, ALUSLTU,  PCSELPLUS4,  1'b0, 1'b0, WBSELRD};
                default:    ctrl = 16'bx;
            endcase
        end
        REGIMM: begin
            case(`RT(Instruction))
                BLTZ:       ctrl = {1'b0, 1'b0, OP0SELRS, OP1SELRT, ALUBLTZ, PCSELBRANCH, 1'b0, 1'b0, WBSELNONE};
                BGEZ:       ctrl = {1'b0, 1'b0, OP0SELRS, OP1SELRT, ALUBGTZ, PCSELBRANCH, 1'b0, 1'b0, WBSELNONE};
                default:    ctrl = 16'bx;
            endcase
        end
        J:          ctrl = {1'b0, 1'b0, 1'bx,     2'bx,        5'bx,     PCSELJMPIMM, 1'b0, 1'b0, WBSELNONE};
        JAL:        ctrl = {1'b0, 1'b0, 1'bx,     OP1SELPC,    ALUPASS1, PCSELJMPIMM, 1'b0, 1'b0, WBSEL31};
        BEQ:        ctrl = {1'b0, 1'b0, OP0SELRS, OP1SELRT,    ALUBEQ,   PCSELBRANCH, 1'b0, 1'b0, WBSELNONE};
        BNE:        ctrl = {1'b0, 1'b0, OP0SELRS, OP1SELRT,    ALUBNE,   PCSELBRANCH, 1'b0, 1'b0, WBSELNONE};
        BLEZ:       ctrl = {1'b0, 1'b0, OP0SELRS, OP1SELRT,    ALUBLEZ,  PCSELBRANCH, 1'b0, 1'b0, WBSELNONE};
        BGTZ:       ctrl = {1'b0, 1'b0, OP0SELRS, OP1SELRT,    ALUBGTZ,  PCSELBRANCH, 1'b0, 1'b0, WBSELNONE};
        ADDI:       ctrl = {1'b0, 1'b0, OP0SELRS, OP1SELSEIMM, ALUADD,   PCSELPLUS4,  1'b0, 1'b0, WBSELRT};
        ADDIU:      ctrl = {1'b0, 1'b0, OP0SELRS, OP1SELSEIMM, ALUADDU,  PCSELPLUS4,  1'b0, 1'b0, WBSELRT};
        SLTI:       ctrl = {1'b0, 1'b0, OP0SELRS, OP1SELSEIMM, ALUSLT,   PCSELPLUS4,  1'b0, 1'b0, WBSELRT};
        SLTIU:      ctrl = {1'b0, 1'b0, OP0SELRS, OP1SELSEIMM, ALUSLTU,  PCSELPLUS4,  1'b0, 1'b0, WBSELRT};
        ANDI:       ctrl = {1'b0, 1'b0, OP0SELRS, OP1SELZEIMM, ALUAND,   PCSELPLUS4,  1'b0, 1'b0, WBSELRT};
        ORI:        ctrl = {1'b0, 1'b0, OP0SELRS, OP1SELZEIMM, ALUOR,    PCSELPLUS4,  1'b0, 1'b0, WBSELRT};
        XORI:       ctrl = {1'b0, 1'b0, OP0SELRS, OP1SELZEIMM, ALUXOR,   PCSELPLUS4,  1'b0, 1'b0, WBSELRT};
        LUI:        ctrl = {1'b0, 1'b0, OP0SELRS, OP1SELZEIMM, ALULUI,   PCSELPLUS4,  1'b0, 1'b0, WBSELRT};
        COP0: begin
            case(`RS(Instruction))
                MFC0:       ctrl = {1'b1, 1'b0, 1'bx, 2'bx, 5'bx, PCSELPLUS4,  1'b0, 1'b0, WBSELRT};
                MTC0:       ctrl = {1'b0, 1'b1, 1'bx, 2'bx, 5'bx, PCSELPLUS4,  1'b0, 1'b0, WBSELNONE};
                default:    ctrl = 16'bx;
            endcase
        end
        LW:         ctrl = {1'b0, 1'b0, OP0SELRS, OP1SELSEIMM, ALUADD,   PCSELPLUS4,  1'b1, 1'b0, WBSELRT};
        SW:         ctrl = {1'b0, 1'b0, OP0SELRS, OP1SELSEIMM, ALUADD,   PCSELPLUS4,  1'b0, 1'b1, WBSELNONE};
        default:    ctrl = 16'bx;
    endcase
end

endmodule
